
## Copyright (C) 2007 - Alfonso Alba
## 
## This file is part of OctavEEG, an EEG processing library for GNU Octave
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##

## Usage: [MPD, ELECPAIRS] = eeg_coherence(DATA, PARAM, FREQ, WMS, ELECPAIRS, ALPHA, NBINS)
##
## Computes significant changes in inter-electrode coherence in the dataset DATA
## at the frequencies given by vector FREQ. Coherence is estimated in time windows of
## length WMS (in ms). PARAM is a parameter structure (as returned by eeg_set_parameters), 
## and ELECPAIRS is an N*2 matrix describing the electrode pairs whose synchrony is
## computed - by default, all electrode pairs are computed by setting
## ELECPAIRS = nchoosek(1:param.nelec, 2). ALPHA and BINS are as described in
## eeg_significance_tt.
##
## Example:
##
##  files = eeg_find_datafiles('data_directory');
##  param = eeg_set_parameters(19, 200, 200);
##  eegdata = eeg_read_dataset(files, param, 128); 
##  mpd = eeg_synchrony_mpd(eegdata, param, 1:10);
##
## See also: eeg_read_dataset, eeg_filter_dataset, eeg_significant_changes

## Author: Alfonso Alba <fac@galia.fc.uaslp.mx>
## Keywords: OctavEEG eeg signal-processing


function [coh, elecpairs] = eeg_coherence(data, param, freq = [], wms = 100, elecpairs = "all", alpha = 0.05, nbins = 256)
  if !ismatrix(data)
    error('first argument must be a 3D or 4D matrix');
  end
  if !isvector(freq)
    error('third argument must be a vector');
  end
  
  if strcmp(elecpairs, "all")
    ne = min(param.nchan, size(data)(ndims(data)));
    elecpairs = nchoosek(1:ne, 2);
  end

  if ndims(data) == 3
    # Unfiltered data - dimensions are (freq, time, elec)
    nt = size(data)(2);
    coh = zeros([length(freq), nt, rows(elecpairs)]);
    for k = 1:length(freq)
      fdata = eeg_filter_dataset(data, param, freq(k), true);
      coh(k, :, :) = eeg_coherence(fdata, param, freq(k), wms, elecpairs, alpha, nbins);
    end
  else
    if ndims(data) == 4    
      # Filtered data - dimensions are (freq, trial, time, elec)
      nt = size(data)(3);
      coh = zeros([size(data)(1), nt, rows(elecpairs)]);
      C = zeros([size(data)(2), nt, rows(elecpairs)]);
      
      # subtract average over time
      nt = size(data)(3);
      data = data - repmat(mean(data, 3), [1, 1, nt, 1]);
      
      w = round(wms * param.samplerate / 2000);
      kernel = ones(1, 2 * w + 1) / (2 * w + 1);
      si = zeros([2 * w, size(data)(2), 2 * w]);
      
      for e = 1:rows(elecpairs)
        XY = data(:, :, :, elecpairs(e, 1)) .* conj(data(:, :, :, elecpairs(e, 2)));
        XX = data(:, :, :, elecpairs(e, 1)) .* conj(data(:, :, :, elecpairs(e, 1)));
        YY = data(:, :, :, elecpairs(e, 2)) .* conj(data(:, :, :, elecpairs(e, 2)));
        Rxy = filter(kernel, 1, postpad(XY, nt + w, 0, 3), si, 3)(:, :, (1+w):(nt+w));
        Rxx = filter(kernel, 1, postpad(XX, nt + w, 0, 3), si, 3)(:, :, (1+w):(nt+w));
        Ryy = filter(kernel, 1, postpad(YY, nt + w, 0, 3), si, 3)(:, :, (1+w):(nt+w));
        C = eeg_subtract_preavg(abs(Rxy) ./ sqrt(abs(Rxx .* Ryy)), param, 3); # C(f,r,t)
        coh(:, :, e) = eeg_average_trials(C, 2);
        printf("Progress: %6.2f         \r", 100 * e / rows(elecpairs));
        fflush(stdout());
      end
           
      printf("Done!                     \n");
      if (alpha > 0) && (nbins > 0)
        if rows(elecpairs) < 2
          coh = eeg_significant_changes(coh', param, alpha, nbins)';
        else
          coh = eeg_significant_changes(coh, param, alpha, nbins);
        end
      end
    end
  end
end
