
## Copyright (C) 2007 - Alfonso Alba
## 
## This file is part of OctavEEG, an EEG processing library for GNU Octave
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##

## Usage: FDATA = eeg_convolve_dataset(DATA, H, VERBOSE, PREFIX)
##
## Filters matrix data along dimension ndims(data) - 1, by convolution with
## the kernel given in H. If VERBOSE is true, print out progress 
## information (by default VERBOSE is true). The optional
## string PREFIX is added to the progress display. The output FDATA 
## is the filtered cell array.
##
## Example:
##
##  files = eeg_find_datafiles('data_directory');
##  eegdata = eeg_read_dataset(files, 19, 128);  # assume 19 channels
##  kernel = eeg_kernel_gabor(10, 2, 512);       # gabor filter tuned at 10 Hz
##                                               # for a sample rate of 512 Hz
##  filtered = eeg_convolve_dataset(eegdata, kernel);
##
## See also: eeg_read_dataset, eeg_kernel_gabor, eeg_kernel_sinusoidal

## Author: Alfonso Alba <fac@galia.fc.uaslp.mx>
## Keywords: OctavEEG eeg signal-processing

function f = eeg_convolve_dataset(data, h, verbose = true, prefix = '')
  f = [];
  if !ismatrix(data)
    error('first argument must be a 2D or 3D matrix');
  end
  if ndims(data) < 2
    error('first argument must be a 2D or 3D matrix');
  end
  if !isvector(h)
    error('second argument must be a vector');
  end
    
  if ndims(data) == 2
    f = eeg_convolve_trial(data, h);
  else
    if ndims(data) == 3
      f = zeros(size(data));
      for i = 1:size(data)(1)
        f(i,:,:) = eeg_convolve_trial(eeg_remove_singletondims(data(i,:,:)), h);
        if verbose
          printf("%s - Processing: %6.2f%%...   \r", prefix, 100 * i / size(data)(1));
          fflush(stdout());
        end
      end
    end
  end
  if verbose
    printf("%s - Done!                        \n", prefix);
  end
end

    
