
## Copyright (C) 2007 - Alfonso Alba
## 
## This file is part of OctavEEG, an EEG processing library for GNU Octave
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##

## Usage: FDATA = eeg_filter_dataset(DATA, PARAM, FREQ, VERBOSE)
##
## Filters the data in matrix DATA along the first dimension (if
## DATA is a 2D matrix) or along the second dimension (if DATA is 3D).
## The tuning frequencies are specified in vector FREQ in Hertz. 
## PARAM is an EEG parameter structure as returned
## by eeg_set_parameters(). If VERBOSE is true, progress information
## is printed on the screen (by default VERBOSE is true). The output
## matrix FDATA is one dimension higher than DATA, where the first
## dimension corresponds to the frequency band.
##
## Example:
##
##  files = eeg_find_datafiles('data_directory');
##  eegdata = eeg_read_dataset(files, 19, 128);  # assume 19 channels
##  eegparam = eeg_set_parameters(19, 200, 200, 'elecpos1020');
##  filtered = eeg_filter_dataset(eegdata, eegparam, 1:20, true);
##
## See also: eeg_read_dataset, eeg_set_parameters

## Author: Alfonso Alba <fac@galia.fc.uaslp.mx>
## Keywords: OctavEEG eeg signal-processing

function f = eeg_filter_dataset(data, param, freq, verbose = true)
  if !ismatrix(data)
    error('first argument must be a 2D or 3D matrix');
  end
  if !isvector(freq)
    error('third argument must be a vector');
  end
  f = zeros([length(freq), size(data)]);
  type = tolower(substr(param.filtertype, 1, 1));
  for i = 1:length(freq)
    switch (type)
      case 'g'
        h = eeg_kernel_gabor(freq(i), param.bandwidth, param.samplerate);
      case 's'
        h = eeg_kernel_sinusoidal(freq(i), param.bandwidth, param.samplerate);
      otherwise
        error('invalid filter type');
    end
    if ndims(data) == 2
      f(i,:,:) = eeg_convolve_dataset(data, h, verbose, sprintf("Filtering at %f Hz", freq(i)));
    else
      if ndims(data) == 3
        f(i,:,:,:) = eeg_convolve_dataset(data, h, verbose, sprintf("Filtering at %f Hz", freq(i)));
      end
    end
  end
end
