
## Copyright (C) 2007 - Alfonso Alba
## 
## This file is part of OctavEEG, an EEG processing library for GNU Octave
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##

## Usage: [IMG, CMAP] = eeg_head_diagram(ELECVAL, MAP, CMAP, HEAD)
##
## Renders a head diagram of the spatial distribution of values in
## vector ELECVAL using the map template MAP, which can be one of
## four things:
##  1) MAP is unspecified. In this case, a voronoi partition for the
##     10-20 system with 19 electrodes is used.
##  2) MAP is a parameter structure (as returned by 
##     eeg_set_parameters) from which a Voronoi partition is constructuted.
##  3) MAP is an indexed image where each pixel specifies the
##     electrode it is assigned to.
##  4) MAP is a 3D array representing a vector image in which pixel
##     is a vector containing the weights of each electrode at that pixel.
## The range of ELECVAL must be [-1,1].
## The optional parameters CMAP and HEAD may specify an alternative color 
## map and head diagram template. The color map must be a 256*3 array in 
## which each row specifies an RGB value (with values between 0 and 1), 
## rows 1-127 correspond to negative ELECVAL values, row 128 is the background 
## color, rows 129-255 correspond to positive values, and row 256 is the head 
## border color. If no output arguments are specified, the resulting image
## will be displayed with eeg_display_image; otherwise, the image and its
## color map are returned.
##
## Example:
##
##  eeg_head_diagram(rand(1,19) * 2 - 1);
##
## See also: eeg_voronoi_partition, eeg_tft_map

## Author: Alfonso Alba <fac@galia.fc.uaslp.mx>
## Keywords: OctavEEG eeg signal-processing

function [img, cmap] = eeg_head_diagram(elecval, map, cmap = [], head = [])
  if isempty(head) 
    [fid, msg] = fopen('head150.raw', 'rb');
    if fid < 0
      error(msg);
    end
    head = fread(fid, [150, 150], 'uchar')';
    fclose(fid);
    center = [80,75];
    radius = 60;
  else
    center = size(head) / 2;
    radius = 0.4 * min(size(head));
  end

  img = head;
  if isempty(cmap)
    # Build colormap: negative values are plotted in green, whereas positive values in red
    # Colors 1:127 go from green to white, 128 = white, 129:255 from white to red, 256 = black
    fade = (linspace(0, 1, 128)(1:127))';
    cmap = [fade, ones(127,1), fade];                             # green to white
    cmap = [cmap; 1, 1, 1];                                       # white
    cmap = [cmap; ones(127,1), flipud(fade), flipud(fade)];       # white to red
    cmap = [cmap; 0, 0, 0];                                       # black
    if nargin() == 0 return; end
  end
 
  if nargin < 2
    map = eeg_set_parameters(19, 0, 0, 'elecpos1020');
  end

  if isstruct(map)
    map = eeg_voronoi_partition(map, length(head), radius, center);
  end
  
  if !ismatrix(map)
    error('invalid second argument - see help for details');
  end
    
  if ndims(map) == 2
    # each element in map indicates the region to which it belongs
    img = (head == 128) .* round(elecval(map) * 127 + 128);     # inner mask
    img = img + (head == 255) * 128;                            # background mask
    img = img + (head == 0) * 256;                              # head border mask
  else
    # each element in map is a weight vector
    w = zeros(size(map)(2:3));
    if columns(elecval) > 1 elecval = elecval'; end             # make sure elecval is a column
    for y = 1:rows(head)
      w(:,:) = map(y, :, :);
      img(y, :) = (w * elecval)';
    end
    img = (head == 128) .* round(img * 127 + 128);               # apply inner mask
    img = img + 128 * (head == 255) + 256 * (head == 0);        # add background and head border
  end
  
  if nargout() == 0
    eeg_display_image(img, cmap);
  end
end

