
## Copyright (C) 2007 - Alfonso Alba
## 
## This file is part of OctavEEG, an EEG processing library for GNU Octave
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##

## Usage: IP = eeg_induced_power(DATA, PARAM, FREQ, ALPHA, NBINS)
##
## Computes the significant changes in induced log-power from dataset DATA
## at the frequency bands specified by vector FREQ. PARAM is a parameter
## structure, as returned by eeg_set_parameters. ALPHA is the significance
## threshold and NBINS is the number of bins used to estimate the prestimulus
## distribution (see eeg_significant_changes for details). By default,
## ALPHA = 0.05 and NBINS = 256.
##
## Example:
##
##  files = eeg_find_datafiles('data_directory');
##  eegdata = eeg_read_dataset(files, 19, 128);  # assume 19 channels
##  param = eeg_set_parameters(19, 200, 200);
##  ip = eeg_induced_power(eegdata, param, 1:10);
##
## See also: eeg_set_parameters, eeg_significant_changes, eeg_read_dataset

## Author: Alfonso Alba <fac@galia.fc.uaslp.mx>
## Keywords: OctavEEG eeg signal-processing


function ip = eeg_induced_power(data, param, freq = [], alpha = 0.05, nbins = 256)
  if !ismatrix(data)
    error('first argument must be a 3D or 4D matrix');
  end
  if !isvector(freq)
    error('third argument must be a vector');
  end
  if ndims(data) == 3
    # Unfiltered data - dimensions are (freq, time, elec)
    ip = zeros([length(freq), size(data)(2:3)]);
    for k = 1:length(freq)
      fdata = eeg_remove_singletondims(eeg_filter_dataset(data, param, freq(k), true));
      amp = eeg_subtract_preavg(eeg_get_amplitude(fdata, 'l'), param);
      ip(k, :, :) = eeg_significant_changes(eeg_average_trials(amp), param, alpha, nbins);
    end
  else
    if ndims(data) == 4
      # Filtered data - dimension are (freq, trial, time, elec)
      ip = zeros(size(data)([1,3,4]));
      amp = eeg_get_amplitude(data, 'l');
      for k = 1:(size(data)(1))
        x = eeg_subtract_preavg(eeg_remove_singletondims(amp(k, :, :, :)), param);
        ip(k, :, :) = eeg_significant_changes(eeg_average_trials(x), param, alpha, nbins);
        if !isempty(freq)
          printf("Processing for %f Hz - Done!\n", freq(k));
          fflush(stdout());
        end
      end
    else
      error('first argument must be a 3D or 4D matrix');
    end
  end
end
