
## Copyright (C) 2007 - Alfonso Alba
## 
## This file is part of OctavEEG, an EEG processing library for GNU Octave
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##

## Usage: H = eeg_kernel_sinusoidal(FREQ, BW, SR)
##
## Returns the impulse response H of a filter with
## sinusoidal frequency response tuned at
## frequency FREQ (in Hertz) with a bandwidth BW (in Hertz)
## and a sample rate SR (also in Hertz). The bandwidth is
## approximately the same as a Gabor filter with the same BW.
##
## Example:
##
##  h = eeg_kernel_sinusoidal(10, 2, 512);
##  y = conv(x, h);

## Author: Alfonso Alba <fac@galia.fc.uaslp.mx>
## Keywords: OctavEEG eeg signal-processing


function h = eeg_kernel_sinusoidal(freq, bw, sr)
  wk = 2 * pi * freq / sr;
  N = 2 * round(2 * sr / (pi * bw)) + 1;        # filter width of approx 4*sw
  bw0 = 2 * pi * (2.4 * bw) / sr;   # 2.4 * bw gives results similar to gabor
  bwk = min(bw0, wk);
  w = linspace(0, 2 * pi, N + 1)(1:N);
  HR = 0.5 * (1 + cos(pi * (w - wk) / bwk));
  HL = 0.5 * (1 + cos(pi * (w - wk) / bw0));
  HR = HR .* (w >= (wk - bwk)) .* (w <= wk);
  HL = HL .* (w > wk) .* (w <= (wk + bw0));
  H = HR + HL;
  h = fftshift(ifft(H));
end

