
## Copyright (C) 2007 - Alfonso Alba
## 
## This file is part of OctavEEG, an EEG processing library for GNU Octave
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##

## Usage: IMG = eeg_multitoposcope(ELECVAL, PARAM, BIGHEADSIZE, SMALLHEADSIZE, INTERP)
##
## Renders a multitoposcopic display as an RGB image IMG. The data is given by
## ELECVAL, which must be a square matrix of size given by PARAM.nchan. The size of 
## the image is BIGHEADSIZE * BIGHEADSIZE * 3, whereas SMALLHEADSIZE determines the 
## size of the head diagrams within the multitoposcope. By default, BIGHEADSIZE = 400 
## and SMALLHEADSIZE = 50. The PARAM argument is a parameter structure as given by
## eeg_set_parameters which specifies the electrode positions. INTERP is the interpolation
## method used to scale the images (see interp2 for details). By default 'nearest'
## interpolation is used.
##
## See also: interp2, eeg_synchrony_mpd, eeg_get_syncmatrix

## Author: Alfonso Alba <fac@galia.fc.uaslp.mx>
## Keywords: OctavEEG eeg signal-processing

function img = eeg_multitoposcope(elecval, param, bigheadsize = 400, smallheadsize = 50, interp = "nearest")
  if !ismatrix(elecval)
    error('first argument must be a square matrix');
  end
  if !issquare(elecval)
    error('first argument must be a square matrix');
  end

  # create white RGB image
  [headtemplate, cmap] = eeg_head_diagram();
  img = eeg_scale_image(eeg_head_diagram(zeros(1, param.nchan)), [bigheadsize, bigheadsize], cmap, interp);
  vmap = eeg_voronoi_partition(param, 150, 60, [85, 75]);
  
  smallheadpos = zeros(param.nchan, 2);
  for e = 1:param.nchan
    smallheadpos(e,:) = [param.elecpos{e}.y, param.elecpos{e}.x];
  end
  t = [0.54 * bigheadsize - smallheadsize / 2, 0.5 * bigheadsize - smallheadsize / 2];
  smallheadpos = 0.43 * (bigheadsize - smallheadsize) * smallheadpos;
  smallheadpos = round(smallheadpos + repmat(t, param.nchan, 1));
  smallheadpos

  for e = 1:rows(elecval)
    head = eeg_head_diagram(elecval(e, :), vmap, cmap, headtemplate);
    rgb = eeg_scale_image(head, [smallheadsize, smallheadsize], cmap, interp);
    y = smallheadpos(e,1):(smallheadpos(e,1) + smallheadsize - 1);
    x = smallheadpos(e,2):(smallheadpos(e,2) + smallheadsize - 1);
    img(y, x, :) = rgb;
  end

  if nargout == 0
    handle = imshow(img);
  end
end

