
## Copyright (C) 2007 - Alfonso Alba
## 
## This file is part of OctavEEG, an EEG processing library for GNU Octave
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##

## Usage: [DATA, ECTRIALS, ECID] = eeg_read_dataset(FILES, NCHAN, ID)
##
## Loads an EEG data set in TrackWalker format. The dataset is stored
## as a cell array DATA{k}{j}, where k denotes an experimental condition
## with identifier ECID{k}, and {j} denotes the trial number. Each element
## of the cell array is a matrix whose columns correspond to the electrode
## signals. The vector ECTRIALS[k] indicates the number of trials read for 
## each experimental condition. The argument FILES is a cell array that 
## contains the data file names, as returned by eeg_find_datafiles, PARAM 
## is a parameter structure as returned by eeg_set_parameters, and ID 
## is an optional parameter which indicates the function to return only those 
## trials whose identifier is ID (in this case, DATA should only be indexed 
## by the trial number). 
##
## Example:
##
##  files = eeg_find_datafiles('data_directory');
##  eegdata = eeg_read_dataset(files, 20, 128);  # assume 20 channels
##
## See also: eeg_find_datafiles, eeg_read_subjectdata, eeg_set_parameters

## Author: Alfonso Alba <fac@galia.fc.uaslp.mx>
## Keywords: OctavEEG eeg signal-processing

function [D, ECTRIALS, ECID, NT] = eeg_read_dataset(files, param, idi = -1)
  if isstruct(param)
    nchan = param.nchan;
  else
    if isscalar(param)
      nchan = param;
    else
      error('second argument must be either an integer or a parameter structure');
    end
  end

  if !isvector(idi)
    error('third argument must be a vector');
  end
  
  [ECTRIALS, ECID, NT] = eeg_read_datasetsize(files, idi);
  D = cell(1, length(ECID));
  s = sum(ECTRIALS);
  for q = 1:length(ECID)
    D{q} = reshape(zeros(s(q), NT * nchan), [s(q), NT, nchan]);
  end
    
  # Loop over all subjects
  for s = 1:files.nfiles
    [d, ectrials, ecid] = eeg_read_subjectdata(files.plgfiles{s}, files.winfiles{s}, nchan, idi);
    
    # Loop over all EC's
    for i = 1:length(ecid)
      q = find(ECID == ecid(i), 1);
      
      # Append EEG trial data
      m = sum(ECTRIALS(1:(s-1),q));
      D{q}((m + 1):(m + ECTRIALS(s,q)), :, :) = d{i};
    end
  end
  
  # Select specific ECID
  if length(D) == 1 
    D = D{1};
  end
end
