
## Copyright (C) 2007 - Alfonso Alba
## 
## This file is part of OctavEEG, an EEG processing library for GNU Octave
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##

## Usage: [DATA, ECTRIALS, ECID, NT] = eeg_read_subjectdata(PLGFILE, WINFILE, NCHAN, IDI)
##
## Reads EEG data from a PLG/WIN file pair in TrackWalker format.
## The files are specified by the arguments PLGFILE and WINFILE,
## while NCHAN indicates the number of recorded channels (electrodes).
## The data is returned as a cell array DATA{k}, where k denotes an 
## experimental condition with identifier ECID(k). Each element of the 
## cell array is a 3-dimensional matrix V(j,t,e) where j is the trial number
## t is the time sample, and e is the electrode. The vector ECTRIALS[k] 
## indicates the number of trials read for each experimental condition.
## NT will contain the number of time samples in each signal in the dataset.
## If the optional parameter IDI is given, only the 3D matrix corresponding
## to that experimental condition identifier is returned. If NCHAN is missing, 
## the function will return an empty DATA set, but it will still return ECTRIALS 
## and ECID, which is useful if one only wants to know the number of trials stored 
## in the files.
##
## Example:
##
##  files = eeg_find_datafiles('data_directory');
##  data = eeg_read_subjectdata(files.plgfiles{1}, files.winfiles{1}, 20);
##
## See also: eeg_find_datafiles, eeg_read_dataset

## Author: Alfonso Alba <fac@galia.fc.uaslp.mx>
## Keywords: OctavEEG eeg signal-processing

function [D, ECTRIALS, ECID, NT] = eeg_read_subjectdata(plg, win, nchan = 0, idi = -1)
  if !isvector(idi)
    error('fourth argument must be a vector');
  end
   
  D = cell(1, length(idi));
  ECTRIALS = zeros(1, length(idi));
  ECID = idi;
  NT = 0;
  
  if isscalar(idi)
    if idi < 0
      idi = 0:255;
      ECID = [];
    end
  end
  
  # Open PLG and WIN files and handle any errors
  [plgfid, msg] = fopen(plg, 'rb', 'ieee-le');
  if plgfid < 0
    error(msg);
  end
  [winfid, msg] = fopen(win, 'rb', 'ieee-le');
  if winfid < 0
    fclose(plgfid);
    error(msg);
  end
  
  # Process WIN file
  while !feof(winfid)
    id = fread(winfid, 1, 'uint8');
    ti = fread(winfid, 1, 'int32');
    tf = fread(winfid, 1, 'int32');
    if (tf - ti + 1) > NT
      NT = tf - ti + 1;
    end
    if !feof(winfid)
      # Check if id is in idi
      if ismember(id, idi)

        # Check if the id is a new one
        q = find(ECID == id, 1);
        if isempty(q)
          ECID = [ECID, id];
          ECTRIALS = [ECTRIALS, 0];
          q = length(ECID);
        end
      
        ECTRIALS(q)++;
      
        if nchan > 0
          # Read EEG trial data 
          fseek(plgfid, (ti - 1) * nchan * 2, SEEK_SET());
          D{q}(ECTRIALS(q),:,:) = fread(plgfid, [nchan, tf - ti + 1], 'int16')';
        end
      end
    end
  end
  
  fclose(plgfid);
  fclose(winfid);
end

