
## Copyright (C) 2007 - Alfonso Alba
## 
## This file is part of OctavEEG, an EEG processing library for GNU Octave
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##

## Usage: SIG = eeg_significance_tt(DATA, NULL, NBINS)
##
## Performs a two-tail significance test of each element in DATA
## against the distribution of the null sample given by NULL. If
## DATA and NULL are matrices, the test is performed column-wise.
## The optional argument NBINS specifies the number of bins used
## to estimate the histogram of NULL (defaults to 256 bins).
##
## Example:
##
## See also:

## Author: Alfonso Alba <fac@galia.fc.uaslp.mx>
## Keywords: OctavEEG eeg signal-processing

function sig = eeg_significance_tt(data, nul, alpha = 0.05, nbins = 256)
  if !ismatrix(data) || !ismatrix(nul)
    error('arguments must be matrices');
  end
  if columns(data) != columns(nul)
    error('arguments must have the same number of columns');
  end
  sig = zeros(size(data));
  for k = 1:columns(data)
    # compute histogram of null sample
    [n,x] = hist(nul(:,k), nbins, 1);
    binwidth = x(2) - x(1);
    
    # compute kernel with bandwidth given by Silverman's rule
    bandwidth = 1.06 * std(nul(:,k)) * rows(nul)^(-0.2);
    bws = bandwidth / binwidth; # bandwidth in samples
    t = (-4 * bws) : (4 * bws);
    kernel = exp(-t.^2 / (2 * bws^2));
    
    # convolve histogram with gaussian kernel
    nn = conv(n, kernel);
    nn = nn / sum(nn);
    xx = linspace(x(1) - 4 * bandwidth, x(length(x)) + 4 * bandwidth, length(nn));
    
    # compute cumulative density
    P = cumsum(nn);
    P0 = P(find(xx >= 0, 1));
    
    # compute upper and lower thresholds
    lalpha = alpha * P0;
    ualpha = 1 - alpha * (1 - P0);
    lt = xx(find(P > lalpha, 1));
    ut = xx(find(P > ualpha, 1));

    # classify significant values
    sig(:,k) = (data(:,k) > ut) - (data(:,k) < lt);
  end
end

  