## Copyright (C) 2007 - Alfonso Alba
## 
## This file is part of OctavEEG, an EEG processing library for GNU Octave
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##

## Usage: SIG = eeg_significant_changes(DATA, PARAM, ALPHA, NBINS)
##
## Classifies the data in DATA according to their statistical significance
## with respect to a null distribution obtained from the pre-stimulus segments.
## Elements in DATA whose p-value is less than ALPHA are classified as -1, 
## elements whose p-value is greater than (1-ALPHA) are classified as +1, and
## all the other values are classified as 0. By default, ALPHA is 0.05. 
## PARAM is a parameter structure, as returned by eeg_set_param, which contains 
## the stimulus time. NBINS is the number of bins used to estimate the null 
## distribution (defaults to 256).
##
## Example:
##
##  files = eeg_find_datafiles('data_directory');
##  eegdata = eeg_read_dataset(files, 19, 128);  # assume 19 channels
##  param = eeg_set_parameters(19, 200, 200);
##  filtdata = eeg_filter_dataset(eegdata, 10, param);
##  ampchanges = eeg_subtract_preavg(eeg_get_amplitude(filtdata), param);
##  sig = eeg_significant_changes(ampchanges, param);
##
## See also: eeg_set_parameters, eeg_subtract_preavg

## Author: Alfonso Alba <fac@galia.fc.uaslp.mx>
## Keywords: OctavEEG eeg signal-processing

function sig = eeg_significant_changes(data, param, alpha = 0.05, nbins = 256)
  if !ismatrix(data)
    error('first argument must be a matrix');
  end
  
  skip = round(3 * param.samplerate / (2 * pi * param.bandwidth)); # filter width
  if skip >= param.tstim / 3
    skip = param.tstim / 3;
  end
  prestim = (1 + skip):(param.tstim - skip);
  
  if ndims(data) < 3
    skip = round(3 * param.samplerate / (2 * pi * param.bandwidth)); # filter width
    sig = eeg_significance_tt(data, data((1 + skip):(param.tstim - skip),:), alpha, nbins);
  else
    sig = zeros(size(data));
    idx.type = "()";
    idx.subs = cell(1, ndims(data));
    for d = 1:ndims(data)
      idx.subs{d} = ":";
    end

    for i = 1:(size(data)(1))
      idx.subs{1} = i;
      x = eeg_significant_changes(eeg_remove_singletondims(subsref(data, idx)), param, alpha, nbins);
      subsasgn(sig, idx, x);
    end
  end
end
