
## Copyright (C) 2007 - Alfonso Alba
## 
## This file is part of OctavEEG, an EEG processing library for GNU Octave
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##

## Usage: [MPD, ELECPAIRS] = eeg_synchrony_mpd(DATA, PARAM, FREQ, ELECPAIRS, ALPHA, NBINS)
##
## Computes significant changes in phase synchronization using in the dataset DATA
## using the magnitude of phase difference (MPD) measure at the frequencies given
## by vector FREQ. PARAM is a parameter structure (as returned by eeg_set_parameters), 
## and ELECPAIRS is an N*2 matrix describing the electrode pairs whose synchrony is
## computed - by default, all electrode pairs are computed by setting
## ELECPAIRS = nchoosek(1:param.nelec, 2). ALPHA and BINS are as described in
## eeg_significance_tt.
##
## Example:
##
##  files = eeg_find_datafiles('data_directory');
##  param = eeg_set_parameters(19, 200, 200);
##  eegdata = eeg_read_dataset(files, param, 128); 
##  mpd = eeg_synchrony_mpd(eegdata, param, 1:10);
##
## See also: eeg_read_dataset, eeg_filter_dataset, eeg_significant_changes

## Author: Alfonso Alba <fac@galia.fc.uaslp.mx>
## Keywords: OctavEEG eeg signal-processing


function [mpd, elecpairs] = eeg_synchrony_mpd(data, param, freq = [], elecpairs = "all", alpha = 0.05, nbins = 256)
  if !ismatrix(data)
    error('first argument must be a 3D or 4D matrix');
  end
  if !isvector(freq)
    error('third argument must be a vector');
  end
  
  if strcmp(elecpairs, "all")
    ne = min(param.nchan, size(data)(ndims(data)));
    elecpairs = nchoosek(1:ne, 2);
  end

  if ndims(data) == 3
    # Unfiltered data - dimensions are (freq, time, elec)
    nt = size(data)(2);
    mpd = zeros([length(freq), nt, rows(elecpairs)]);
    for k = 1:length(freq)
      fdata = eeg_filter_dataset(data, param, freq(k), true);
      mpd(k, :, :) = eeg_synchrony_mpd(fdata, param, freq(k), elecpairs, alpha, nbins);
    end
  else
    if ndims(data) == 4    
      # Filtered data - dimensions are (freq, trial, time, elec)
      nt = size(data)(3);
      mpd = zeros([size(data)(1), nt, rows(elecpairs)]);
      data = eeg_get_phase(data);
      for e = 1:rows(elecpairs)
          # phasedif has dimensions (freq, trial, time)
          phasedif = 1 - abs(data(:, :, :, elecpairs(e,1)) - data(:, :, :, elecpairs(e, 2))) / pi;
          x = eeg_subtract_preavg(phasedif, param, 3);
          mpd(:, :, e) = eeg_average_trials(x, 2);
      end
      
%      bws = round(100 * param.samplerate / 4000); # bandwidth in samples
%      w = 4 * bws;
%      t = (-w):w;
%      kernel = exp(-t.^2 / (2 * bws^2));
%      si = zeros([2 * w, rows(elecpairs)]);
%      mpd = postpad(mpd, nt + w, 0, 2);
%      mpd = filter(kernel, 1, mpd, si, 2)(:, (1+w):(nt+w), :);

      mpd = eeg_significant_changes(mpd, param, alpha, nbins);
    end
  end
end
