
## Copyright (C) 2007 - Alfonso Alba
## 
## This file is part of OctavEEG, an EEG processing library for GNU Octave
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##

## Usage: [TFT, CMAP] = eeg_tf_map(DATA, PARAM, FREQ, DELTAT, CELLSIZE, INTERP, VERBOSE)
##
## Renders a time-frequency (TF) map of the matrix DATA, which represents a normalized 
## EEG measure at frequency FREQ(k), where the columns are time series corresponding to
## the different electrodes. PARAM is a parameter structure as returned by 
## eeg_set_parameters. DELTAT specifies the width in samples of the time-cells in which 
## the TFT map is divided (default is 10 samples). INTERP specifies the interpolating
## method used for resampling the image (default is 'nearest'). Progress information
## will be displayed if VERBOSE is true (which is the default). If no output arguments
## are specified, the function will use eeg_display_image to display the TFT map;
## otherwise, the image and its color map are returned.
##
## Example:
##
##  files = eeg_find_datafiles('data_directory');
##  eegdata = eeg_read_dataset(files, 19, 128);  # assume 19 channels
##  param = eeg_set_parameters(19, 200, 200, 'elecpos1020');
##  induced = eeg_induced_power(eegdata, param, 1:20);
##  eeg_tft_map(induced, param, 1:20);
##
## See also: eeg_head_diagram, eeg_display_image, imresize

## Author: Alfonso Alba <fac@galia.fc.uaslp.mx>
## Keywords: OctavEEG eeg signal-processing


function tft = eeg_tf_map(data, param, freq = [])
  if !ismatrix(data)
    error('first argument must be a 2D matrix');
  end
  if ndims(data) != 2
    error('first argument must be a 2D matrix');
  end

  fade = (linspace(0, 1, 128)(1:127))';
  cmap = [fade, ones(127,1), fade];                             # green to white
  cmap = [cmap; 1, 1, 1];                                       # white
  cmap = [cmap; ones(127,1), flipud(fade), flipud(fade)];       # white to red
  cmap = [cmap; 0, 0, 0];                                       # black
  
  nt = columns(data);
  data = round(data * 127 + 128);
  tft = ind2rgb(data(:,(param.tstim):nt), cmap);
    
  if nargout == 0
    handle = imshow(tft);
    set(handle, 'xdata', [0, (nt - param.tstim) * 1000 / param.samplerate]);
    xlabel('Time (ms)');
    if !isempty(freq)
      set(handle, 'ydata', [min(freq)-0.5, max(freq)+0.5]);
      ylabel('Frequency (Hz)');
    end
  end
end
