
## Copyright (C) 2007 - Alfonso Alba
## 
## This file is part of OctavEEG, an EEG processing library for GNU Octave
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##

## Usage: [TFT, CMAP] = eeg_tft_hist(DATA, PARAM, FREQ, DELTAT, CELLSIZE, INTERP, VERBOSE)
##
## Renders a time-frequency-topography (TFT) histogram of the dataset DATA, which must be
## a unidimensional cell array of matrices. The matrix DATA{k} represents a normalized 
## EEG measure at frequency FREQ(k), where the columns are time series corresponding to
## the different electrodes. PARAM is a parameter structure as returned by 
## eeg_set_parameters. DELTAT specifies the width in samples of the time-cells in which 
## the TFT map is divided (default is 10 samples). INTERP specifies the interpolating
## method used for resampling the image (default is 'linear'). Progress information
## will be displayed if VERBOSE is true (which is the default). If no output arguments
## are specified, the function will use eeg_display_image to display the TFT map;
## otherwise, the image and its color map are returned.
##
## Example:
##
##  files = eeg_find_datafiles('data_directory');
##  eegdata = eeg_read_dataset(files, 19, 128);  # assume 19 channels
##  param = eeg_set_parameters(19, 200, 200, 'elecpos1020');
##  induced = eeg_induced_power(eegdata, param, 1:20);
##  eeg_tft_map(induced, param, 1:20);
##
## See also: eeg_head_diagram, eeg_display_image, imresize

## Author: Alfonso Alba <fac@galia.fc.uaslp.mx>
## Keywords: OctavEEG eeg signal-processing


function tft = eeg_tft_hist(data, param, freq = [], deltat = 10, cellsize = 32, interp = 'linear', verbose = true)
  if !ismatrix(data)
    error('first argument must be a 3D matrix');
  end
  if ndims(data) != 3
    error('first argument must be a 3D matrix');
  end
  
  nt = size(data)(2);
  ncx = floor((nt - param.tstim) / deltat) - 1;
  ncy = size(data)(1);

  if verbose printf("Initializing...                \r"); end

  # create colormap
  colors = [ 1, 1, 0; 0, 1, 0; 0, 0.5, 1; 1, 0.5, 0.25; 1, 0, 0 ];
  cmap = interp1(linspace(1, 127, rows(colors)), colors, 1:127);
  cmap = [flipud(cmap); 1, 1, 1; cmap; 0, 0, 0];

  # create white RGB image
  headtemplate = eeg_head_diagram();
  tft = ones([ncy * cellsize, ncx * cellsize, 3]);
  vmap = eeg_voronoi_partition(param, 150, 60, [85, 75]);
%  vmap = eeg_interpolation_table(param, 150, 10, 60, [85, 75]);
  n = 0;
  tot = ncy * ncx;
  tfty = 1:cellsize;
  
  for y = 1:ncy
    tftx = 1:cellsize;
    t = (param.tstim + 1):(param.tstim + deltat);
    for x = 1:ncx
      head = eeg_head_diagram(mode(eeg_remove_singletondims(data(y, t, :))), vmap, cmap, headtemplate);
      rgb = eeg_scale_image(head, [cellsize, cellsize], cmap, interp);
      tft(tfty, tftx, :) = rgb;
      tftx = tftx + cellsize;
      t = t + deltat;
      if verbose
        printf("Progress: %6.2f%%   \r", 100 * (++n) / tot);
        fflush(stdout());
      end
    end
    tfty = tfty + cellsize;
  end
  
  if verbose printf("Done!                \n"); end
    
  if nargout == 0
    n = floor(size(tft)(2) * 3 / 4);
    if n > size(tft)(1)
      img = postpad(tft, n, 1);
    else
      img = tft;
    end

    handle = imshow(img);
    set(handle, 'xdata', [0, (nt - param.tstim) * 1000 / param.samplerate]);
    xlabel('Time (ms)');
    if !isempty(freq) 
      set(handle, 'ydata', [min(freq), max(freq)]);
      ylabel('Frequency (Hz)');
    end
  end
end
